"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = require("node-fetch");
const interfaces_1 = require("./interfaces");
let k = '00000000000000000000000000000000';
let p = '00000000000000000000000000000000';
const K_TAG = '<input id="k" type="hidden" value="';
const P_TAG = '<input id="p" type="hidden" value="';
const headers = {
    Host: 'burghquayregistrationoffice.inis.gov.ie',
    'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:62.0) Gecko/20100101 Firefox/62.0',
    Accept: 'application/json, text/javascript, */*; q=0.01',
    Referer: 'https://burghquayregistrationoffice.inis.gov.ie/Website/AMSREG/AMSRegWeb.nsf/AppSelect?OpenForm',
    'X-Requested-With': 'XMLHttpRequest',
    Cookie: '_ga=GA1.3.1378869052.1536830642; _gid=GA1.3.1590764695.1537783699; cookieconsent_status=dismiss',
    Connection: 'keep-alive',
};
function buildUrl(type, category, subcategory) {
    return `https://burghquayregistrationoffice.inis.gov.ie/Website/AMSREG/AMSRegWeb.nsf/(getApps4DTAvailability)?readform&&cat=${category}&sbcat=${subcategory}&typ=${type}`;
}
function getSlots(json) {
    return typeof json.slots === 'string' ? JSON.parse(json.slots) : json.slots;
}
function isSessionExpired(slots) {
    return __awaiter(this, void 0, void 0, function* () {
        return slots !== undefined ? slots : yield getNewTokens();
    });
}
function logMessage(slots) {
    const message = getLogMessage(slots);
    console.info(new Date(), message);
    return slots;
}
function buildReturn({ slots, type, category, subcategory }) {
    return {
        status: 'success',
        type,
        category,
        subcategory,
        data: slots
    };
}
function getNewTokens() {
    return __awaiter(this, void 0, void 0, function* () {
        let keep = true;
        while (keep) {
            try {
                const response = yield node_fetch_1.default('https://burghquayregistrationoffice.inis.gov.ie/Website/AMSREG/AMSRegWeb.nsf/AppSelect?OpenForm');
                const page = yield response.text();
                const { newK, newP } = getKayAndPiTokensFromPage(page);
                k = newK;
                p = newP;
                keep = false;
            }
            catch (err) { }
        }
        console.info('Tokens expired, restarting with new tokens!');
        return [];
    });
}
function getLogMessage(slots) {
    return slots.length > 0 ?
        `Dates found: ${JSON.stringify(slots)}` :
        'No dates available at the moment';
}
function getKayAndPiTokensFromPage(page) {
    const kStartIndex = page.lastIndexOf(K_TAG) + K_TAG.length;
    const pStartIndex = page.lastIndexOf(P_TAG) + P_TAG.length;
    const newK = page.substring(kStartIndex, kStartIndex + k.length);
    const newP = page.substring(pStartIndex, pStartIndex + p.length);
    return {
        newK,
        newP
    };
}
function checkSlotsAvailability(type = interfaces_1.Types.New, category = interfaces_1.Categories.Study, subcategory = interfaces_1.Subcategories.All) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
            const URL_CHECK_SLOTS = buildUrl(type, category, subcategory);
            const requestResult = yield node_fetch_1.default(URL_CHECK_SLOTS + `&k=${k}` + `&p=${p}`, {
                method: 'GET',
                headers
            })
                .then(res => res.json())
                .then(getSlots)
                .then(isSessionExpired)
                .then(logMessage)
                .then(slots => buildReturn({ slots, type, category, subcategory }));
            return requestResult;
        }
        catch (error) {
            return {
                status: 'error',
                type,
                category,
                subcategory,
                error
            };
        }
    });
}
exports.checkSlotsAvailability = checkSlotsAvailability;
